
/*

MySlider v.1.2

by Andchir

http://wdevblog.net.ru/

*/

(function ( $ ) {
    
    $.mySlider = {
        
        timer: {},
        effect_duration: 0,
        
        /* default layer`s options */
        defaultLayerOptions: {
            text: '',
            width: 150,
            height: 150,
            pos_left: 0,
            pos_top: 0,
            padding: 10,
            font: '14px Arial, Helvetica, sans-serif',
            background: 'background-color:transparent',
            text_color: '#000',
            opacity: 1,
            link: '',
            effect: '',
            visible: true
        },
        
        /* stop all effects */
        effectsStop: function(slide){
            
            var effectIndex = 0;
            
            $('.myslider_layer',slide).each(function(){
                
                var layer = $(this);
                var effect = $(this).data('effect');
                if (typeof(effect) != 'undefined' && effect.length > 0) {
                    clearTimeout($.mySlider.timer['ef'+effectIndex]);
                    //layer.finish();
                    effectIndex++;
                }
                
            });
            
            return effectIndex;
            
        },
        
        /* effects prepare */
        effectsPrepare: function(slide){
            
            var effectIndex = 0;
            $.mySlider.effect_duration = 0;
            
            $('.myslider_layer',slide).each(function(){
                
                var layer = $(this);
                var effect = $(this).data('effect');
                if (typeof(effect) != 'undefined' && effect.length > 0) {
                    layer
                    .addClass('effect')
                    .hide({effect:'fade', duration:0});
                    effectIndex++;
                }
                
            });
            
            return effectIndex;
            
        },
        
        /* get effect data from string */
        parseEffectData: function(string){
            var out = {};
            var effect_arr = string.length > 0 ? string.split(':') : ['fade'];
            out.effect = effect_arr[0];
            out.duration = 1500;
            out.direction = '';
            if (effect_arr[1] && isNaN(effect_arr[1])) {
                out.direction = effect_arr[1];
            }
            if (effect_arr[1] && !isNaN(effect_arr[1])) {
                out.duration = parseInt(effect_arr[1]);
            }
            if (effect_arr[2] && !isNaN(effect_arr[2])) {
                out.duration = parseInt(effect_arr[2]);
            }
            return out;
        },
        
        /* effects initialization */
        effectsInit: function(slide){
            
            var effectIndex = 0;
            
            $($('.myslider_layer',slide).get().reverse()).each(function(){
                
                var layer = $(this);
                var effect = $(this).data('effect');
                
                if (typeof(effect) != 'undefined' && effect.length > 0) {
                    
                    $.mySlider.runEffect(layer, effect, effectIndex);
                    effectIndex++;
                    
                }
                
            });
            
        },
        
        /* run effect */
        runEffect: function(elem, effect, effectIndex){
            
            var effect_data = $.mySlider.parseEffectData(effect);
            
            $.mySlider.timer['ef'+effectIndex] = setTimeout(function(){
                
                var opts = {easings: 'swing'};
                opts.direction = effect_data.direction;
                if (effect_data.effect === "scale" ) {
                    opts.percent = 100;
                }
                elem.show(effect_data.effect, opts, effect_data.duration);
                
            }, $.mySlider.effect_duration);
            
            $.mySlider.effect_duration += effect_data.duration;
            
        },
        
        /* next */
        next: function(e, slider_wrapper){
            
            if (e != null && $(e.target).size() > 0) {
                var slider_wrapper = $(e.target).parents('div.myslider_wrapper');
                $.mySlider.clearAutoplay(slider_wrapper);
            }
            if (typeof(slider_wrapper) == 'undefined') {
                var slider_wrapper = $('div.myslider_wrapper:first');
            }else if (!slider_wrapper.is('.myslider_wrapper')) {
                var slider_wrapper = slider_wrapper.children('.myslider_wrapper');
            }
            
            var slide_index = slider_wrapper.parent().data('slide-index');
            var slides_count = $('li.myslider_slide',slider_wrapper).size();
            
            if (slide_index + 1 == slides_count) {
                slide_index = -1;
            }
            
            $.mySlider.slideTo(null, slider_wrapper, (slide_index+1));
            
        },
        
        /* prev */
        prev: function(e, slider_wrapper){
            
            if (e != null && $(e.target).size() > 0) {
                var slider_wrapper = $(e.target).parents('div.myslider_wrapper');
                $.mySlider.clearAutoplay(slider_wrapper);
            }
            if (typeof(slider_wrapper) == 'undefined') {
                var slider_wrapper = $('div.myslider_wrapper:first');
            }else if (!slider_wrapper.is('.myslider_wrapper')) {
                var slider_wrapper = slider_wrapper.children('.myslider_wrapper');
            }
            
            var slide_index = slider_wrapper.parent().data('slide-index');
            
            if(slide_index == 0){
                var slides_count = $('li.myslider_slide',slider_wrapper).size();
                slide_index = slides_count;
            }
            
            $.mySlider.slideTo(null, slider_wrapper, (slide_index-1));
            
        },
        
        /* slide to index */
        slideTo: function(e, slider_wrapper, slide_index){
            
            if (e != null && $(e.target).size() > 0) {
                if ($(e.target).is('.active')) return false;
                var slider_wrapper = $(e.target).parents('div.myslider_wrapper');
                $.mySlider.clearAutoplay(slider_wrapper);
            }
            if (typeof slide_index == 'undefined') {
                var slide_index = $(e.target).prevAll().size();
            }
            if (typeof slider_wrapper == 'undefined') {
                var slider_wrapper = $('div.myslider_wrapper:first');
            }
            
            var direction = slider_wrapper.is('.horizontal') ? 'horizontal' : 'vertical';
            var old_slide_index = slider_wrapper.parent().data('slide-index');
            slider_wrapper.parent().data({'slide-index':slide_index});
            var slide_container = $('ul.myslider_container_b',$('div.myslider_container',slider_wrapper));
            var slide_step = direction == 'horizontal' ? slide_container.parent().width() : slide_container.parent().height();
            
            $('div.myslider_nav_next,div.myslider_nav_prev',slider_wrapper).unbind('click');
            $('li',$('ul.myslider_index_nav',slider_wrapper)).unbind('click');
            $.mySlider.slideMark(slider_wrapper, slide_index);
            
            //stop old effects
            $.mySlider.effectsStop($('li.myslider_slide',slide_container).eq(old_slide_index));
            //prepare new effects
            var effects_count = $.mySlider.effectsPrepare($('li.myslider_slide',slide_container).eq(slide_index));
            
            slide_container
            .animate({
                "margin-left": (direction == 'horizontal' ? (0 - (slide_step * slide_index))+"px" : null),
                "margin-top": (direction == 'vertical' ? (0 - (slide_step * slide_index))+"px" : null)
                }, 1000, function() {
                
                $.mySlider.bindNavEvents(slider_wrapper);
                
                $.mySlider.effectsInit($('li.myslider_slide',slide_container).eq(slide_index));
                
            });
            
            return false;
        },
        
        /* clear autoplay timer */
        clearAutoplay: function(slider_wrapper){
            
            var sl_index = slider_wrapper.parent().data('index');
            var play_delay = slider_wrapper.parent().data('autoPlay');
            
            if (play_delay > 0) {
                clearInterval($.mySlider.timer[sl_index]);
                $.mySlider.timer[sl_index] = setInterval(function(){
                    $.mySlider.next(null,slider_wrapper.parent());
                }, (play_delay*1000));
            }
            
        },
        
        /* mark current slide */
        slideMark: function(container, slide_index){
            
            $('li',$('ul.myslider_index_nav',container))
            .removeClass('active')
            .eq(slide_index)
            .addClass('active');
            
        },
        
        /* bind click events to slider navigation */
        bindNavEvents: function(slider_wrapper){
            
            if ($('div.myslider_nav_next',slider_wrapper).size() > 0) {
                $('div.myslider_nav_next',slider_wrapper).bind('click', $.mySlider.next);
                $('div.myslider_nav_prev',slider_wrapper).bind('click', $.mySlider.prev);
            }
            
            $('li',$('ul.myslider_index_nav',slider_wrapper)).bind('click', $.mySlider.slideTo);
            
        },
        
        /* show navigation */
        showNav: function(e){
            
            var slider_wrapper = $(e.target).parents('div.myslider_wrapper');
            var direction = slider_wrapper.is('.horizontal') ? 'horizontal' : 'vertical';
            var slides_count = $('li.myslider_slide',slider_wrapper).size();
            
            if (slides_count > 1) {
                
                if (e.type == 'mouseenter') {
                    $('div.myslider_nav_prev',slider_wrapper)
                    .css({
                        left: (direction == 'horizontal' ? '30px' : '50%'),
                        bottom: (direction == 'vertical' ? '30px' : '50%'),
                        opacity: 0
                    })
                    .next('div.myslider_nav_next')
                    .css({
                        right: (direction == 'horizontal' ? '30px' : '50%'),
                        top: (direction == 'vertical' ? '30px' : '50%'),
                        opacity: 0
                    });
                }
                
                if (direction == 'horizontal') {
                    
                    $('div.myslider_nav_prev',slider_wrapper)
                    .stop()
                    .animate({
                        opacity: (e.type == 'mouseenter' ? 1 : 0),
                        left: (e.type == 'mouseenter' ? "-=15" : 0)
                    }, 500);
                    
                    $('div.myslider_nav_next',slider_wrapper)
                    .stop()
                    .animate({
                        opacity: (e.type == 'mouseenter' ? 1 : 0),
                        right: (e.type == 'mouseenter' ? "-=15" : 0)
                    }, 500);
                    
                }else if (direction == 'vertical') {
                    
                    $('div.myslider_nav_prev',slider_wrapper)
                    .stop()
                    .animate({
                        opacity: (e.type == 'mouseenter' ? 1 : 0),
                        bottom: (e.type == 'mouseenter' ? "-=15" : 0)
                    }, 500);
                    
                    $('div.myslider_nav_next',slider_wrapper)
                    .stop()
                    .animate({
                        opacity: (e.type == 'mouseenter' ? 1 : 0),
                        top: (e.type == 'mouseenter' ? "-=15" : 0)
                    }, 500);
                    
                }
                
                
            }
            
        }
        
        
    }
    
    $.fn.mySlider = function( options, optval ) {
        
        if (typeof options == 'string') {
            if ($.inArray(options, ['next','prev','slideTo']) > -1) {
                $.mySlider[options](null, this, optval);
            }
            return this;
        }
        
        if((!options || !options.items) && typeof(sliderConstructData)=='object'){
            var options = !options ? sliderConstructData : $.extend({}, sliderConstructData, options);
        }
        
        var settings = $.extend({},{
            slider_width: 700,
            slider_height: 250,
            direction: 'horizontal',//horizontal, vertical
            showArrows: true,
            autoPlay: true,
            delay: 6,
            items: []
        }, options );
        
        if (settings.items.length == 0 && $('ul',this).size() == 0) {
            return this;
        }
        
        return this.each(function() {
            
            var root = $(this);
            var sl_index = $('div.myslider_wrapper').size();
            var json_mode = settings.items.length > 0;
            var items_count = json_mode ? settings.items.length : $('li',root).size();
            
            root
            .data({
                'index': sl_index,
                'slide-index': 0,
                'autoPlay': (settings.autoPlay ? settings.delay : 0)
            });
            
            //wrapper
            if (json_mode) {
                root
                .append('<div class="myslider_wrapper"><div class="myslider_container"><ul class="myslider_container_b"></ul></div></div>');
            }else{
                $('ul',root).wrap('<div class="myslider_wrapper"><div class="myslider_container"></div></div>');
                if (!$('ul',root).is('.myslider_container_b')) {
                    $('ul',root).addClass('myslider_container_b');
                }
            }
            
            root
            .find('div.myslider_wrapper, div.myslider_container')
            .css({width: settings.slider_width+'px', height: settings.slider_height+'px'})
            .filter('div.myslider_wrapper')
            .addClass(settings.direction);
            
            var slider_container = $('div.myslider_container',root);
            
            $('ul.myslider_container_b',slider_container)
            .css({
                width: (settings.direction == 'horizontal' ? (settings.slider_width * items_count + 10) : settings.slider_width ) + 'px',
                height: (settings.direction == 'horizontal' ? settings.slider_height : (settings.slider_height * items_count + 10)) + 'px'
            });
            
            //slides
            for(var index=0; index<items_count; index++){
                
                if (json_mode) {
                    $('ul.myslider_container_b',slider_container).append('<li class="myslider_slide"></li>');
                    var slide_item = $('li.myslider_slide',slider_container).eq(index);
                }else{
                    var slide_item = $('li',slider_container).eq(index);
                    if (!slide_item.is('.myslider_slide')) {
                        slide_item.addClass('myslider_slide');
                    }
                    $('*',slide_item).addClass('myslider_layer');
                }
                
                slide_item
                .css({
                    width: settings.slider_width+'px',
                    height: settings.slider_height+'px',
                    'float': (settings.direction == 'horizontal' ? 'left' : 'none')
                });
                
                if (json_mode) {
                    
                    //layers
                    for(var i in settings.items[index]){
                        
                        var layer_opts = $.extend({}, $.mySlider.defaultLayerOptions, settings.items[index][i]);
                        
                        //background
                        var bg_arr = layer_opts.background.split(':');
                        if (bg_arr[0]=='background-color') {
                            var background_color = bg_arr[1]=='transparent' ? '' : bg_arr[1];
                        }
                        
                        if (typeof(layer_opts.link) != 'undefined' && layer_opts.link.length > 0) {
                            var layer_html = '<a class="myslider_layer" href="'+layer_opts.link+'" target="_blank"><span class="inner">'+settings.items[index][i].text+'</span></a>';
                        }else{
                            var layer_html = '<div class="myslider_layer"><div class="inner">'+layer_opts.text+'</div></div>';
                        }
                        slide_item.prepend(layer_html);
                        
                        var layer = $('.myslider_layer',slide_item).eq(0);
                        
                        layer
                        .css({
                            width: layer_opts.width,
                            height: layer_opts.height,
                            left: layer_opts.pos_left+'px',
                            top: layer_opts.pos_top+'px',
                            font: layer_opts.font,
                            background: bg_arr[1],
                            color: layer_opts.text_color,
                            opacity: layer_opts.opacity,
                            display: layer_opts.visible ? 'block' : 'none'
                        })
                        .data('effect',layer_opts.effect)
                        .find('.inner')
                        .css({padding: layer_opts.padding+'px'});
                        
                    }
                    
                }
                
                if (index == 0) {
                    $.mySlider.effectsPrepare(slide_item);
                    $.mySlider.effectsInit(slide_item);
                }
                
            }
            
            if (items_count > 1) {
                
                //prev / next navigation
                if (settings.showArrows) {
                    
                    slider_container
                    .prepend('<div class="myslider_nav_prev"></div><div class="myslider_nav_next"></div>')
                    .find('div.myslider_nav_prev,div.myslider_nav_next')
                    .css({opacity: 0});
                    
                    slider_container
                    .bind('mouseenter mouseleave',$.mySlider.showNav);
                    
                }
                
                //index navigation
                var index_nav = '<ul class="myslider_index_nav">';
                for(var i=0; i < items_count; i++){
                    index_nav += '<li'+(i==0 ? ' class="active"' : '')+'>'+(i+1)+'</li>';
                }
                index_nav += '</ul>';
                
                slider_container.prepend(index_nav);
                
                $.mySlider.bindNavEvents(root);
                
                //autoPlay
                if (settings.autoPlay) {
                    $.mySlider.timer[sl_index] = setInterval(function(){
                        $.mySlider.next(null,root);
                    }, (settings.delay*1000));
                }
                
            }
            
        });
        
    }
    
    
}( jQuery ));

